
/* 
 * File:   GSLPermutation.h
 * Author: skidmarks
 *
 * Created on December 29, 2017, 2:04 PM
 */

#ifndef GSLPERMUTATION_H
#define GSLPERMUTATION_H

# include <gsl/gsl_permutation.h>
# include <gsl/gsl_permute.h>
# include <gsl/gsl_permute_matrix.h>
# include <gsl/gsl_permute_vector.h>

# include "GSLData.h"
# include "GSLMatrix.h"
# include "GSLVector.h"

using namespace std;
namespace gsl {

   class GSLPermutation : public GSLData {
   public:
      GSLPermutation(size_t size);
      GSLPermutation(size_t size, size_t calloc);
      GSLPermutation(const GSLPermutation& orig);
      virtual ~GSLPermutation();
      virtual string toString() {return "";};
    
   /******************************************************
    *                   Initialization                   *
    ******************************************************/
      virtual void init()                                      { gsl_permutation_init((gsl_permutation*)block());};
      virtual int  swap(const size_t i, const size_t j)        { return gsl_permutation_swap((gsl_permutation*)block(), i, j);};

   /******************************************************
    *                      Accessing                     *
    ******************************************************/
   virtual size_t get(const size_t i)                          { return gsl_permutation_get((gsl_permutation*)block(), i);};
   
   /******************************************************
    *                      Copying                       *
    ******************************************************/
   virtual int memcpy(GSLPermutation& dest)                   { return gsl_permutation_memcpy((gsl_permutation*)dest.block(), (gsl_permutation*)block());};
   
   /******************************************************
    *                    Properties                      *
    ******************************************************/
//   virtual size_t* data() override                            { return (size_t*)block();};
   virtual int     valid()                                    { return gsl_permutation_valid((gsl_permutation*)block());};
   
   /******************************************************
    *               Permutation Functions                *
    ******************************************************/
   virtual void reverse()                                     { gsl_permutation_reverse((gsl_permutation*)block());};
   virtual int  inverse(GSLPermutation& inv)                  { return gsl_permutation_inverse((gsl_permutation*)inv.block(), (gsl_permutation*)block());};
   virtual int  next()                                        { return gsl_permutation_next((gsl_permutation*)block());};
   virtual int  prev()                                        { return gsl_permutation_prev((gsl_permutation*)block());};
   
   /******************************************************
    *                Applying Permutations               *
    ******************************************************/
   virtual int  permute(double * array, size_t stride, size_t size) { return gsl_permute((size_t*)data(), array,stride, size);};
   virtual int  permute_inverse(double * array, size_t stride, size_t size) { return gsl_permute_inverse((size_t*)data(), array,stride, size);};
   virtual int  permute_vector(GSLVector& v)                 { return gsl_permute_vector((gsl_permutation*)block(), (gsl_vector*)v.block());};
   virtual int  permute_vector_inverse(GSLVector& v)         { return gsl_permute_vector_inverse((gsl_permutation*)block(), (gsl_vector*)v.block());};
   virtual int  permute_matrix(GSLMatrix& A)                 
                        { return gsl_permute_matrix((gsl_permutation*)block(), (gsl_matrix*)A.block());};
   virtual int  permute_mul(const GSLPermutation& pa, const GSLPermutation& pb) 
         { return gsl_permutation_mul((gsl_permutation*)block(), (gsl_permutation*)pa.block(), (gsl_permutation*)pb.block() );};
   
   /******************************************************
    *            Permutations in Cyclic Form             *
    ******************************************************/
   virtual int  linear_to_canonical(GSLPermutation& q)      { return gsl_permutation_linear_to_canonical((gsl_permutation*)q.block(), (gsl_permutation*)block());};
   virtual int  canonical_to_linear(GSLPermutation& q)      { return gsl_permutation_canonical_to_linear((gsl_permutation*)block(), (gsl_permutation*)q.block());};
   virtual size_t  perutation_inversions()                  { return gsl_permutation_inversions((gsl_permutation*)block());};
   virtual size_t  linear_cycles()                          { return gsl_permutation_linear_cycles((gsl_permutation*)block());};
   virtual size_t  canonical_cycles()                       { return gsl_permutation_canonical_cycles((gsl_permutation*)block());};
 
   private:

   }; // class GSLPermutation
} // namespace gsl
#endif /* GSLPERMUTATION_H */

